/**************************************************************************************************************************************************
 *       Copyright  2015 Texas Instruments Incorporated - http://www.ti.com/                                                                      *
 ***************************************************************************************************************************************************
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met: *
 *                                                                                                                                                 *
 *    Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.                 *
 *                                                                                                                                                 *
 *    Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the        *
 *    documentation and/or other materials provided with the distribution.                                                                         *
 *                                                                                                                                                 *
 *    Neither the name of Texas Instruments Incorporated nor the names of its contributors may be used to endorse or promote products derived      *
 *    from this software without specific prior written permission.                                                                                *
 *                                                                                                                                                 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT          *
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     *
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT         *
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY    *
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE      *
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                                                           *
 **************************************************************************************************************************************************/


#include "msp430.h"
#include "stdint.h"
#include "swif.h"
#include "main.h"

volatile unsigned int error=0, counter1kdata=0, baudrate;
volatile unsigned char buttonDebounce, mode, statusLED=1;

#ifdef WRITE_FRAM
#pragma PERSISTENT(data)
#pragma location = 0xC200
volatile unsigned char data = 0x00;

#else
volatile unsigned char data=0x00;
#endif

//=================================================================================================================================================
//Main
//=================================================================================================================================================

int main(void)
{
	WDTCTL = WDTPW | WDTHOLD;                           // Stop Watchdog

	initPorts();
	initUCS();
	initCompE();
	baudrate=BAUDRATE;
	config_USCIA0_UART(baudrate);
	prepareRX();
	mode = STANDBY_MODE;

	while(1)
	{
		switch(mode)
		{
		case SHUTDOWN_MODE:
			UCA0CTLW0 |= UCSWRST;
			CECTL1 &= ~CEON;                          // Turn Off ComparatorE
			UCA0IE &= ~(UCRXIE + UCTXIE);
			P4OUT &= ~BIT5;                           // Turn red LED Off
			P4OUT &= ~BIT6;                           // Turn green LED Off
			P4OUT |= BIT7;                            // Turn yellow LED On
			statusLED=1;
			data = 0x00;
			counter1kdata=0;
			error = 0;
			__bis_SR_register(LPM4_bits | GIE);
			__no_operation();                         // For debug
			initUCS();
			initCompE();
			config_USCIA0_UART(baudrate);
			prepareRX();
			break;

		case STANDBY_MODE:
		    initCompE();
		    config_USCIA0_UART(baudrate);
			prepareRX();
			P4OUT &= ~BIT5;                           // Turn red LED Off
			P4OUT |= BIT6;                            // Turn green LED On
			P4OUT &= ~BIT7;                           // Turn yellow LED Off
			statusLED=1;
			data = 0x00;
			counter1kdata=0;
			error = 0;
#ifdef BREAK_SYNC_SEQUENCE
			UCA0CTLW0 &= ~UCDORM;
#endif
			__bis_SR_register(LPM0_bits | GIE);
			//__bis_SR_register(LPM3_bits | GIE);
			__no_operation();                         // For debug
			__delay_cycles(CPU_CLK*13);
			break;

		case ACTIVE_RX_TX_MODE:
			if(statusLED)
			{
				if(counter1kdata > 1024)
				{
					counter1kdata = 0;
					if(statusLED)
					{
						P4OUT ^= BIT6;                 // Toggle green LED
					}
					__no_operation();                  // For debug
				}
			}
			if (error > 0)
			{
				P4OUT |= BIT5;                        // Turn red LED On
			}
			break;

		default:
			break;
		}
	}
}

/*************************************************************************************************************************************************
 *  initPorts
 **************************************************************************************************************************************************/
/*!
 * @brief Initialize the GPIO pins on the MSP430FR5969
 *
 * This function is called once to configure the GPIO pins on the system.
 *
 * @return  none
 **************************************************************************************************************************************************/
void initPorts (void)
{
	// Configure GPIO
	P3DIR |= BIT5;
	P1OUT = BIT2;
	P1REN |= BIT1;
#ifdef MSP430_IS_ON_ISOLATED_SIDE
	P1DIR &= ~BIT0;					// PG Signal
	P1DIR |= (BIT1 + BIT2 + BIT3 + BIT4 + BIT5 + BIT6 + BIT7);
#else
	P1DIR |= (BIT0 + BIT1 + BIT2 + BIT3 + BIT4 + BIT5 + BIT6 + BIT7);
#endif
	P2OUT = 0x00;
	P2DIR |= (BIT2 + BIT3 + BIT4 + BIT5 + BIT6 + BIT7);  // P2.1 input
	P3OUT = BIT5;
	P3DIR |= 0xFF;
	P4OUT = 0x00;
	P4DIR |= 0xFF;
	PJOUT = 0x00;
	PJDIR |=0xFF;

	P2SEL1 |= (BIT0 + BIT1);                          // UCA0TXD UART operation
	P1DIR  |=BIT1;                                    //COUT

	//Initialize buttons
	P4DIR &= ~(BIT2 + BIT3 + BIT4);
	P4OUT |= (BIT2 + BIT3 + BIT4);
	P4REN |= (BIT2 + BIT3 + BIT4);
	P4IES = (BIT2 + BIT3 + BIT4);                     // Hi/Lo edge
	PM5CTL0 &= ~LOCKLPM5;	                          // Disable the GPIO power-on default high-impedance mode to activate
	// previously configured port settings
	P4IFG = 0;                               		 // Clear all P4 interrupt flags
	P4IE = (BIT2 + BIT3 + BIT4);                     // interrupt enabled

	buttonDebounce = 1;
	P4OUT |= BIT6;    // Turn green LED On
}

/*************************************************************************************************************************************************
 *  initUCS
 **************************************************************************************************************************************************/
/*!
 * @brief MSP430 System Clock Init 8MHz
 *
 * This function is for MSP430FR5969 & might need modifications for other MSP430s
 *
 * @return  none
 **************************************************************************************************************************************************/
void initUCS(void)
{
	// Startup clock system with max DCO setting ~8MHz
	CSCTL0_H = CSKEY >> 8;                                          // Unlock clock registers
	CSCTL1 = DCOFSEL_3 | DCORSEL;                                   // Set DCO to 8MHz
	CSCTL2 = SELA__LFXTCLK | SELS__DCOCLK | SELM__DCOCLK;
	CSCTL3 = DIVA__1 | DIVS__1 | DIVM__1;                           // Set all dividers
	PJSEL0 = BIT4 | BIT5;                     // For XT1
	CSCTL4 &= ~LFXTOFF;
	do
	{
		CSCTL5 &= ~LFXTOFFG;                    // Clear XT1 fault flag
		SFRIFG1 &= ~OFIFG;
	}while (SFRIFG1&OFIFG);                   // Test oscillator fault flag

	CSCTL0_H = 0;                                                   // Lock CS registers
}

/*************************************************************************************************************************************************
 *  startWDT
 **************************************************************************************************************************************************/
/*!
 * @brief start watchdog timer
 *
 * WDT as 250ms interval counter
 *
 * @return  none
 **************************************************************************************************************************************************/

void startWDT()
{
	//WDT as 250ms interval counter
	SFRIFG1 &= ~WDTIFG;
	WDTCTL = WDT_ADLY_250;                           // WDT 250ms, ACLK, interval timer
	SFRIE1 |= WDTIE;                                 // Enable WDT interrupt
}

/*
 * eUSCI Interrupt Service Routine
 * Handles eUSCI_A0 interrupts
 */
#pragma vector=USCI_A0_VECTOR
__interrupt void USCI_A0_ISR(void)
{
	switch(__even_in_range(UCA0IV, USCI_UART_UCTXCPTIFG))
	{
	case USCI_NONE: break;
	case USCI_UART_UCRXIFG:
		counter1kdata++;
		if (data == UCA0RXBUF)
		{
			data++;
		}
		else
		{
			error++;
		}
		if (mode == STANDBY_MODE)
		{
			mode = ACTIVE_RX_TX_MODE;
#ifdef BREAK_SYNC_SEQUENCE
          UCA0CTLW0 |= UCDORM;
#endif
          error=0;
          data=0;
          __bic_SR_register_on_exit(LPM4_bits);
          __delay_cycles(CPU_CLK*20);                   //wait some time for slave to be ready
          sendByte_RX(data++);
		}
		else if (mode == ACTIVE_RX_TX_MODE)
		{
			__delay_cycles(CPU_CLK*5);                   //delay needed for slave to be ready if baudrate below 20000 baud
			sendByte_RX(data++);
		}
		break;
	case USCI_UART_UCTXIFG: break;
	case USCI_UART_UCSTTIFG: break;
	case USCI_UART_UCTXCPTIFG: break;
	}
}

/*
 * PORT4 Interrupt Service Routine
 * Handles button press interrupt
 */
#pragma vector = PORT4_VECTOR
__interrupt void PORT4_ISR(void)
{
	if (buttonDebounce == 1)
	{
		buttonDebounce = 2;
		startWDT();
		switch(__even_in_range(P4IV, P4IV_P4IFG7))
		{
		case P4IV_NONE   : break;
		case P4IV_P4IFG0 : break;
		case P4IV_P4IFG1 : break;
		case P4IV_P4IFG2 :
			switch(mode)
			{
			case SHUTDOWN_MODE:
				mode = STANDBY_MODE;
				__bic_SR_register_on_exit(LPM4_bits);
				break;
			case STANDBY_MODE:
				mode = ACTIVE_RX_TX_MODE;
				prepareTX();
				sendByte_RX(data);
#ifdef BREAK_SYNC_SEQUENCE
          UCA0CTLW0 |= UCDORM;
#endif
				__bic_SR_register_on_exit(LPM4_bits);
				break;
			default:
				mode = SHUTDOWN_MODE;
				__bic_SR_register_on_exit(LPM4_bits);
				break;
			}
			break;
			case P4IV_P4IFG3 :
				statusLED ^= BIT0;
				P4OUT &= ~BIT5;    // Turn red LED Off
				P4OUT &= ~BIT6;    // Turn green LED Off
				P4OUT &= ~BIT7;    // Turn yellow LED Off
				break;
			case P4IV_P4IFG4 :
                statusLED ^= BIT0;
                P4OUT &= ~BIT5;    // Turn red LED Off
                P4OUT &= ~BIT6;    // Turn green LED Off
                P4OUT &= ~BIT7;    // Turn yellow LED Off
				break;
			case P4IV_P4IFG5 : break;
			case P4IV_P4IFG6 : break;
			case P4IV_P4IFG7 : break;
		}
	}
	else
	{
		P4IFG = 0;
	}
}

/*
 * WDT Interrupt Service Routine
 * Handles WDT interrupt
 */
#pragma vector=WDT_VECTOR
__interrupt void WDT_ISR(void)
{
	if (buttonDebounce == 2)
	{
		buttonDebounce = 1;
		SFRIFG1 &= ~WDTIFG;
		SFRIE1 &= ~WDTIE;
		WDTCTL = WDTPW + WDTHOLD;
	}
}



